;*************************** TI protocol subroutines ************************
;*
;* Title:               TI_prot: subroutines for TI protocol
;* Version:		1.2
;* Last updated:        00.10.01
;* Target:		AT90Sxxxx (All AVR devices)
;*
;* Support E-mail:      rlievin@mail.com
;*
;* Code Size            :?? words
;* Low Register Usage   :1 (r18)
;* High Register Usage  :3 (r16, r17, r18)
;* Interrupt Usage	:none
;*
;* DESCRIPTION
;*
;* This unit provide some subroutines to communicate with TI calculators.
;* This is a part of the AVRlink project.
;* Vversion 1.1: I added a new routine: ti_spy for the AVRanalyzer 
;* project.
;* Version  1.2: some improvements/tests about the tempo routine.
;*
;* AVRlink, (c) 2000 by Romain Lievin
;* This code is placed under GPL license
;*
;**************************************************************************


;***** Bit positions in the port B *****

.equ    RED     =0              ;Red wire	(port B)
.equ    WHITE   =1              ;White wire	(port B)

;***** Register declarations (don't use r30=Z) *****

.def    bitcnt  =r16            ;bit counter
.def    scratch =r17            ;a register for scratch
.def    ti_reg	=r18		;register for TI routines

;**************************************************************************
;*
;* "check"
;*
;* This subroutine checks if the 2 lines are equals to 1
;*
;* Total number of words        : 4
;* Total number of cycles       : 3 to 7
;* Low register usage           : 0 (bit T=red & white)
;* High register usage          : 0
;*
;**************************************************************************

check:  
	clt			;clear the T bit
	sbis    PINB,RED        ;if red wire=0 then return
        ret
        sbis    PINB,WHITE      ;if white wire=0 then return
        ret
	set			;if red=1 & white=1, set the T bit
	ret

;**************************************************************************
;*
;* "ti_put"
;*
;* This subroutine transmits the byte stored in the "ti_reg" register
;* This subroutines resets the watchdog when it starts.
;*
;* Total number of words        : 24
;* Total number of cycles       : ?? (without check)
;* Low register usage           : 1 (ti_reg)
;* High register usage          : 2 (ti_reg, bitcnt)
;*
;**************************************************************************

wti_put:	
	wdr			;reset the watchdog
ti_put:
	rcall	check		;if red=0 or white=0, return
	brts	ti_put1
	ret	
ti_put1:clc                     ;clear carry
        ldi     bitcnt,8        ;8 bits to send
nextb:	ror	ti_reg        	;get next bit
	brcc	tbit_1
;
tbit_0:	
	sbi	DDRB,WHITE	;white=0
r0poll:	sbic	PINB,RED	;wait that red=0
	rjmp	r0poll
	rcall	tempoN		;not compulsory
	cbi	DDRB,WHITE	;white=1	
r1poll:	sbis	PINB,RED	;wait that red=1
	rjmp	r1poll
	rjmp	end_tx		;go to counter
;
tbit_1:	
	sbi	DDRB,RED	;red=0
w0poll:	sbic	PINB,WHITE	;wait that white=0
	rjmp	w0poll
	rcall	tempoN		;not compulsory
	cbi	DDRB,RED	;red=1	
w1poll:	sbis	PINB,WHITE	;wait that white=1
	rjmp	w1poll
	rjmp	end_tx		;go to counter
;
end_tx:	
	rcall	tempoN		;not compulsory
	dec	bitcnt		;if not all bit sent
	brne	nextb		;send next
	ret			;end

;**************************************************************************
;*
;* "ti_get"
;*
;* This subroutine receives the byte which is stored in the "ti_reg" register
;* This subroutines resets the watchdog when it starts.
;*
;* Total number of words        : 30
;* Total number of cycles       : ??
;* Low register usage           : 1 (ti_reg)
;* High register usage          : 2 (ti_reg, bitcnt)
;*
;**************************************************************************

wti_get:	
	wdr			;reset the watchdog
ti_get:
	ldi	bitcnt,8	;8 bits to receive
	clc			;clear carry
ti_get1:
	sbis    PINB,RED        ;wait that red or white=0
        rjmp	ti_get2
        sbic    PINB,WHITE
	rjmp	ti_get1
;
ti_get2:
	sbic	PINB,RED	;if red=1, jump
	rjmp	gbit_1
;
gbit_0:	
	clc			;clear carry
	ror	ti_reg		;shift bit into ti_rx
	rcall	tempoN		;not compulsory
	sbi	DDRB,WHITE	;white=0
t0poll:	sbis	PINB,RED	;wait that red=1
	rjmp	t0poll
	rcall	tempoN		;not compulsory
	cbi	DDRB,WHITE	;white=1
	rjmp	end_rx
;
gbit_1:	
	sec			;set carry
	ror	ti_reg		;shift bit into ti_rx
	rcall	tempoN		;not compulsory
	sbi	DDRB,RED	;red=0
t1poll:	sbis	PINB,WHITE	;wait that white=1
	rjmp	t1poll
	rcall	tempoN		;not compulsory
	cbi	DDRB,RED	;red=1
	rjmp	end_rx
;
end_rx:	
	rcall	tempoN		;a small tempo (REQUIRED)
	rcall	tempoN
	dec	bitcnt		;if not all bit sent
	brne	ti_get1		;send next
	ret			;end

;**************************************************************************
;*
;* "ti_spy"
;*
;* This subroutine monitors the Red & White lines and when a TI byte is
;* transmited on the lines, it decodes and stores in the "ti_reg" register
;* This subroutines resets the watchdog when it starts.
;*
;* Total number of words        : ??
;* Total number of cycles       : ??
;* Low register usage           : 1 (ti_reg)
;* High register usage          : 2 (ti_reg, bitcnt)
;*
;**************************************************************************

ti_spy:	
	ldi	bitcnt,0
	mov	ti_reg,bitcnt	;clear the register
	ldi	bitcnt,8	;8 bits to receive
	clc			;clear carry
ti_spy1:
	sbis    PINB,RED        ;wait that red or white=0
	rjmp	ti_spy2		;
	sbic    PINB,WHITE
	rjmp	ti_spy1		;
	
;
ti_spy2:
	sbic	PINB,RED	;if red=0, go on
	rjmp	sbit_1		;else jump
;
; red = 0:
;
sbit_0:	
	sbic	PINB,WHITE	;wait that white=0
	rjmp	sbit_0
 	clc			;clear carry
	ror	ti_reg		;shift bit into ti_reg
w3poll:	sbis	PINB,RED	;wait that red=1
	rjmp	w3poll
r3poll:	sbis	PINB,WHITE	;wait that white=1
	rjmp	r3poll
	rjmp	end_spy
;
; white = 0
;
sbit_1:	
	sbic	PINB,RED	;wait that red=0
	rjmp	sbit_1
	sec			;set carry
	ror	ti_reg		;shift bit into ti_reg
r4poll:	sbis	PINB,WHITE	;wait that white=1
	rjmp	r4poll
w4poll:	sbis	PINB,RED	;wait that red=1
	rjmp	w4poll
	rjmp	end_spy
;
end_spy:	
	dec	bitcnt		;if not all bit sent
	brne	ti_spy1		;get next
	ret			;end

;**************************************************************************
;*
;* "sync"
;*
;* This subroutine waits that a previous RS232 byte has been sent
;*
;* Total number of words        : 3
;* Total number of cycles       : depend of RS232 routines
;* Low register usage           : 0
;* High register usage          : 0
;*
;**************************************************************************

sync:	
	sbis	USR,UDRE	;check if register is empty (AT90S2313) or
;	sbrc	u_status,TD	;wait that the previous RS232 byte (AT90S1200)
	rjmp	sync		;has been sent
	ret

;**************************************************************************
;*
;* "tempo"
;*
;* This subroutine generates a delay. The total execution time is function
;* of scratch (1 microsecond at 12 MHz by default)
;*
;* Total number of words        : 4
;* Total number of cycles       : 2+3*scratch
;* Low register usage           : 0
;* High register usage          : 1 (scratch)
;*
;**************************************************************************

tempoN:	
	ldi	scratch,4	;scratch=4 -> 1 microsecond
tempo:	dec	scratch
	brne	tempo
	ret

	;; Tips
;
;output mode only (DDBn = 1)
;	sbi	PORTn	S=1
;	cbi	PORTn	S=0
;ouput mode and monitored input (PORTBn = 0)
;	cbi	DDRn	S=1/E
;	sbi	DDRn	S=0